﻿/** 
 *   Copyright 2008 Olivian Daniel Tofan
 *   Additions made in 2010 by Robert Tomek
 * 
 *   This file is part of WLPGZenphotoPlugin.
 *
 *   WLPGZenphotoPlugin is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Xml;
using System.Net;

namespace WLPGZenphotoPlugin
{
    public partial class ConfigForm : Form
    {
        private XmlDocument sessionXml;
        private XmlDocument persistXml;
        private bool sesEditMode;
        private ConfigPanel privReturnPanel;

        // Error strings.
        private static readonly string sessionAddError = "You must specify at least session name and host";
        private static readonly string sessionAddErrorTitle = "Add session error";
        private static readonly string sessionExistsError = "A ssesion with name {0} already exists!";
        private static readonly string sessionExistsErrorTitle = "Session exists error";
        private static readonly string sessionTestSuccess = "Connection successful";
        private static readonly string sessionTestSuccessTitle = "Test connection";
        private static readonly string sessionTestFailure = "Connection unsuccessful";
        private static readonly string sessionTestFailureTitle = "Test connection";
        private static readonly string initializePubPanelFailure = "Failed to initialize the publishing panel with the following error:\r\n\t{0}.";
        private static readonly string initializePubPanelFailureTitle = "Initialization Failure";
        private static readonly string loginFailure = "Invalid username or password";
        private static readonly string loginFailureTitle = "Invalid login";
        private static readonly string generalFailure = "A problem has occurred in this application:\r\n\r\n\t{0}.";
        private static readonly string generalFailureTitle = "Unexpected error";
        private static readonly string applicationThreadException = "A problem has occurred in this application:\r\n\r\n\t{0}.";
        private static readonly string applicationThreadExceptionTitle = "Unexpected error";

        // Strings for Form designer label to add resolution support
        private string photoSizeOriginalText;
        private string photoSize2048Text;
        private string photoSize1600Text;
        private string photoSize1280Text;
        private string photoSize1024Text;
        private string photoSize640Text;

        /// <summary>
        /// This is the main entry point called from PublishPlugin.LaunchPluginConfig.
        /// </summary>
        /// <param name="parent">The window to be parented to.</param>
        /// <param name="sessionXml">The XmlDocument containing session specific information for this publish session.</param>
        /// <param name="persistXml">The XmlDocument containing persistent information for this plugin.</param>
        /// <returns>True if the user clicked Publish, otherwise false.</returns>
        public static bool GetSessionInfo(IWin32Window parent, XmlDocument sessionXml, XmlDocument persistXml)
        {
            ConfigForm pf = new ConfigForm();
            pf.sessionXml = sessionXml;
            pf.persistXml = persistXml;
            return pf.ShowDialog(parent) == DialogResult.OK;
        }

        // Used in ShowPanel below. Enumerates all of the panels that 
        // exist in the configuration form.
        private enum ConfigPanel
        {
            Cover,
            Session,
            Publish,
            Upload,
            PrivacyPolicy
        };

        /// <summary>
        /// A short and sweet subclassed Label that enables the transparent 
        /// background color
        /// </summary>
        private class TransLabel : Label
        {
            public TransLabel()
                : base()
            {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            }
        }

        /// <summary>
        /// A short and sweet subclassed LinkLabel that enables the 
        /// transparent background color
        /// </summary>
        private class TransLinkLabel : LinkLabel
        {
            public TransLinkLabel()
                : base()
            {
                this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            }
        }

        private ConfigForm()
        {
            // Establish an exception handler for this class.
            Application.ThreadException += new ThreadExceptionEventHandler(ConfigForm_ThreadException);
            InitializeComponent();
        }

        /// <summary>
        /// Handles any unhandled exceptions thrown by the ConfigureForm by popping a dialog with the
        /// error message and then closing ConfigureForm.
        /// </summary>
        private void ConfigForm_ThreadException(object sender, ThreadExceptionEventArgs e)
        {
            MessageBox.Show(string.Format(ConfigForm.applicationThreadException, e.Exception.Message),
                            ConfigForm.applicationThreadExceptionTitle,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
            Log.Error(e.Exception);
            // Fake that the user hit cancel and close ConfigureForm.
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void ConfigForm_Load(object sender, EventArgs e)
        {
            if (XmlHelper.PersistHasSessions(this.persistXml))
            {
                ShowPublishPanel();
            }
            else
            {
                // No session found, prompt the user to create a session.
                XmlHelper.PersistInitializeDocument(this.persistXml);
                ShowSessionPanel(false);
            }
        }

        /// <summary>
        /// Internal function used to switch which panel is visible.
        /// </summary>
        private void ShowPanel(ConfigPanel panel)
        {
            this.SuspendLayout();

            this.coverPanel.Hide();
            this.sesPanel.Hide();
            this.pubPanel.Hide();
            this.uplPanel.Hide();
            this.priPanel.Hide();
            
            switch (panel)
            {
                case ConfigPanel.Cover:
                    this.AcceptButton = this.coverNextButton;
                    this.CancelButton = this.coverCancelButton;
                    this.coverPanel.Show();
                    this.coverNextButton.Focus();
                    break;
                case ConfigPanel.Session:
                    this.AcceptButton = this.sesNextButton;
                    this.CancelButton = this.sesCancelButton;
                    this.sesPanel.Show();
                    this.sesUrlTextBox.Focus();
                    break;
                case ConfigPanel.Publish:
                    this.AcceptButton = this.pubNextButton;
                    this.CancelButton = this.pubCancelButton;
                    this.pubPanel.Show();
                    this.pubNextButton.Focus();
                    break;
                case ConfigPanel.Upload:
                    this.AcceptButton = this.uplPublishButton;
                    this.CancelButton = this.uplCancelButton;
                    this.uplPanel.Show();
                    this.uplPublishButton.Focus();
                    break;
                case ConfigPanel.PrivacyPolicy:
                    this.AcceptButton = this.priCloseButton;
                    this.CancelButton = null;
                    this.priPanel.Show();
                    this.priCloseButton.Focus();
                    break;
            }

            this.ResumeLayout();
        }
        
        ///////////////////////////////////////////////////////////////////////
        // Cover Panel functions
        ///////////////////////////////////////////////////////////////////////
        
        private void coverNextButton_Click(object sender, EventArgs e)
        {
            this.coverNextButton.Enabled = false;
            this.coverNextButton.Refresh();

            try
            {
                ShowPublishPanel();
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format(ConfigForm.initializePubPanelFailure, ex.Message),
                                ConfigForm.initializePubPanelFailureTitle,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
            finally
            {
                this.coverNextButton.Enabled = true;
            }
        }

        ///////////////////////////////////////////////////////////////////////
        //Session Panel functions
        ///////////////////////////////////////////////////////////////////////

        private void ShowSessionPanel(bool edit)
        {
            sesEditMode = edit;
            InitSessionPanel();
            ShowPanel(ConfigPanel.Session);
        }

        /// <summary>
        /// Internal function used to initialize the Session panel.
        /// </summary>
        private void InitSessionPanel()
        {
            string name;
            XmlHelper.PersistLoadSelectedSession(this.persistXml, out name);
            Session session;
            XmlHelper.PersistLoadSession(this.persistXml, name, out session);
            sesNameTextBox.Enabled = !sesEditMode;
            if (sesEditMode)
            {
                sesTitleLabel.Text = "Edit Zenphoto session";
                sesNameTextBox.Text = session.Name;
                sesUrlTextBox.Text = session.Url;
                sesUserTextBox.Text = session.Username;
                sesPassTextBox.Text = session.Password;
            }
            else
            {
                sesTitleLabel.Text = "Add Zenphoto session";
                sesNameTextBox.Text = "";
                sesUrlTextBox.Text = "";
                sesUserTextBox.Text = "";
                sesPassTextBox.Text = "";
            }
        }

        private Session getSession()
        {
            if (string.IsNullOrEmpty(sesNameTextBox.Text.Trim()) ||
                   string.IsNullOrEmpty(sesUrlTextBox.Text.Trim()))
            {
                MessageBox.Show(ConfigForm.sessionAddError,
                        ConfigForm.sessionAddErrorTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                return null;
            }

            if (!sesEditMode && XmlHelper.PersistExistsSession(this.persistXml, sesNameTextBox.Text.Trim()))
            {
                MessageBox.Show(string.Format(ConfigForm.sessionExistsError, sesNameTextBox.Text.Trim()),
                        ConfigForm.sessionExistsErrorTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                return null;
            }

            Session session = new Session(sesNameTextBox.Text.Trim(),
                                            sesUrlTextBox.Text.Trim());
            session.Username = sesUserTextBox.Text.Trim();
            session.Password = sesPassTextBox.Text;

            return session;
        }

        private void sesNextButton_Click(object sender, EventArgs e)
        {
            this.sesNextButton.Enabled = false;
            this.sesNextButton.Refresh();

            try
            {
                Session session = getSession();
                if (session == null)
                {
                    this.sesNextButton.Enabled = true;
                    return;
                }

                XmlHelper.PersistUpdateSession(this.persistXml, session);
                XmlHelper.PersistStoreSelectedSession(this.persistXml, session.Name);

                ShowPublishPanel();

                session = this.pubSessionComboBox.SelectedItem as Session;
                if (session != null)
                {
                    //TogglePublishButton(session);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format(ConfigForm.generalFailure, ex.Message),
                                ConfigForm.generalFailureTitle,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
            finally
            {
                this.sesNextButton.Enabled = true;
            }
        }

        private void sesTestButton_Click(object sender, EventArgs e)
        {
            this.sesTestButton.Enabled = false;
            this.sesTestButton.Refresh();
            try
            {
                Session session = getSession();
                if (session == null)
                {
                    this.sesTestButton.Enabled = true;
                    return;
                }

                bool result = ZenphotoHelper.ZenphotoLogin(session);
                if (result)
                {
                    MessageBox.Show(ConfigForm.sessionTestSuccess,
                        ConfigForm.sessionTestSuccessTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(ConfigForm.sessionTestFailure,
                        ConfigForm.sessionTestFailureTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                 MessageBox.Show(ConfigForm.sessionTestFailure,
                        ConfigForm.sessionTestFailureTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                 Log.Error(ex);
            }
            finally
            {
                this.sesTestButton.Enabled = true;
            }
        }

        ///////////////////////////////////////////////////////////////////////
        //Publish Panel functions
        ///////////////////////////////////////////////////////////////////////

        private void ShowPublishPanel()
        {
            InitPublishPanel();
            ShowPanel(ConfigPanel.Publish);
        }

        /// <summary>
        /// Internal function used to initialize the Publish panel.
        /// </summary>
        private void InitPublishPanel()
        {
            string name;
            XmlHelper.PersistLoadSelectedSession(this.persistXml, out name);

            // Populating the sessions combo box will automatically populate
            // the rest of the UI.
            PopulateSessions();
            SelectSession(name);

        }

        /// <summary>
        /// Internal function used to populate the Account ComboBox.
        /// </summary>
        private void PopulateSessions()
        {
            this.pubSessionComboBox.Items.Clear();
            List<Session> sessions;
            XmlHelper.PersistLoadSessions(this.persistXml, out sessions);
            foreach (Session session in sessions)
            {
                this.pubSessionComboBox.Items.Add(session);
            }

            // If we weren't able to add any accounts to the ComboBox, something has gone wrong.
            if (this.pubSessionComboBox.Items.Count == 0)
            {
                throw new Exception("No sessions found");
            }

            this.pubSessionComboBox.Items.Add("Add Zenphoto session...");
        }

        /// <summary>
        /// Internal function used to select the Session in the combobox that 
        /// has the specified name.
        /// </summary>
        private void SelectSession(string name)
        {
            for (int i = 0; i < this.pubSessionComboBox.Items.Count - 1; i++)
            {
                Session item = this.pubSessionComboBox.Items[i] as Session;
                if (item.Name == name)
                {
                    this.pubSessionComboBox.SelectedIndex = i;
                }
            }
        }

        /// <summary>
        /// This event is fired when the user clicks the Publish button. 
        /// </summary>
        private void pubNextButton_Click(object sender, EventArgs e)
        {
            this.pubNextButton.Enabled = false;
            this.pubNextButton.Refresh();

            Session session = pubSessionComboBox.SelectedItem as Session;

            if (session != null) 
            {
                // Copy the session
                Session newSession = new Session(session);
                newSession.Username = pubUsernameTextBox.Text.Trim();
                newSession.Password = pubPasswordTextBox.Text;
                // Make sure to save the selected user in the xml files!
                XmlHelper.SessionStoreZenphotoSession(this.sessionXml, newSession);
                XmlHelper.PersistStoreSelectedSession(this.persistXml, newSession.Name);

                bool loggedin = ZenphotoHelper.ZenphotoLogin(newSession);
                if (loggedin)
                {
                    ShowUploadPanel();
                }
                else
                {
                    MessageBox.Show(ConfigForm.loginFailure,
                        ConfigForm.loginFailureTitle,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                    this.pubNextButton.Enabled = true;
                }
            }
        }

        /// <summary>
        /// This event is fired when the user clicks on the privacy policy 
        /// page link.
        /// </summary>
        private void pubPrivacyLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            privReturnPanel = ConfigPanel.PrivacyPolicy;
            ShowPanel(ConfigPanel.PrivacyPolicy);
        }

        
        private void pubRemoveLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Session session = this.pubSessionComboBox.SelectedItem as Session;
            if (session != null)
            {
                XmlHelper.PersistRemoveSession(this.persistXml, session.Name);
                XmlHelper.PersistStoreSelectedSession(this.persistXml, "");

                this.pubSessionComboBox.Items.RemoveAt(this.pubSessionComboBox.SelectedIndex);
                this.pubSessionComboBox.SelectedIndex = 0; // If there are no 
                // more accounts, the first item will be Add Session... and 
                // the selection change event handler will navigate to the 
                // first pane.
            }
        }

        private void pubSessionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Session session = this.pubSessionComboBox.SelectedItem as Session;
            if (session != null)
            {
                // Do logic to see if publish button should be enabled.
                //TogglePublishButton(account);
                pubUsernameTextBox.Text = session.Username;
                pubPasswordTextBox.Text = session.Password;
            }
            else
            {
                ShowSessionPanel(false);
            }
        }

        private void pubEditLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Session session = this.pubSessionComboBox.SelectedItem as Session;
            if (session != null)
            {
                XmlHelper.PersistStoreSelectedSession(this.persistXml, session.Name);
                ShowSessionPanel(true);
            }
        }

        ///////////////////////////////////////////////////////////////////////
        //Upload Panel functions
        ///////////////////////////////////////////////////////////////////////

        private void ShowUploadPanel()
        {
            InitUploadPanel();
            ShowPanel(ConfigPanel.Upload);
        }

        private void InitUploadPanel()
        {
            Session session;
            XmlHelper.SessionLoadZenphotoSession(sessionXml, out session);

            PopulateAlbums(session);
            uplAlbumAutoCheckBox.Checked = true;
        }

        private void PopulateAlbums(Session session)
        {
            this.pubSessionComboBox.Items.Clear();
            List<Album> albums;
            ZenphotoHelper.ZenphotoGetAlbums(session, out albums);
            foreach (Album album in albums)
            {
                this.uplAlbumsComboBox.Items.Add(album);
            }

            // If we weren't able to add any accounts to the ComboBox, something has gone wrong.
            if (this.uplAlbumsComboBox.Items.Count == 0)
            {
                throw new Exception("No albums found");
            }
            uplAlbumsComboBox.SelectedIndex = 0;

            //initialize text
            this.photoSizeOriginalText = (string)this.pubPhotoSizeComboBox.Items[0];
            this.photoSize2048Text = (string)this.pubPhotoSizeComboBox.Items[1];
            this.photoSize1600Text = (string)this.pubPhotoSizeComboBox.Items[2];
            this.photoSize1280Text = (string)this.pubPhotoSizeComboBox.Items[3];
            this.photoSize1024Text = (string)this.pubPhotoSizeComboBox.Items[4];
            this.photoSize640Text = (string)this.pubPhotoSizeComboBox.Items[5];
            this.pubPhotoSizeComboBox.SelectedIndex = 3;
        }

        private void uplPublishButton_Click(object sender, EventArgs e)
        {
            this.uplPublishButton.Enabled = false;
            this.uplPublishButton.Refresh();

            int photoSize = 0;
            if (this.pubPhotoSizeComboBox.SelectedItem.ToString() == this.photoSize2048Text)
            {
                photoSize = 2048;
            }
            else if (this.pubPhotoSizeComboBox.SelectedItem.ToString() == this.photoSize1600Text)
            {
                photoSize = 1600;
            }
            else if (this.pubPhotoSizeComboBox.SelectedItem.ToString() == this.photoSize1280Text)
            {
                photoSize = 1280;
            }
            else if (this.pubPhotoSizeComboBox.SelectedItem.ToString() == this.photoSize1024Text)
            {
                photoSize = 1024;
            }
            else if (this.pubPhotoSizeComboBox.SelectedItem.ToString() == this.photoSize640Text)
            {
                photoSize = 640;
            }
            XmlHelper.SessionStoreDimensions(this.sessionXml, photoSize, photoSize);

            Album parentAlbum = this.uplAlbumsComboBox.SelectedItem as Album;

            UploadData uploadData = new UploadData();
            uploadData.parentAlbum = parentAlbum;
            uploadData.isNewAlbum = this.uplAlbumNewCheckBox.Checked;
            uploadData.isPublic = this.uplAlbumPublishCheckBox.Checked;
            if (uploadData.isNewAlbum)
            {
                string newAlbumName = this.uplAlbumNameTextBox.Text.Trim();
                string newAlbumFolder = this.uplAlbumFolderTextBox.Text.Trim();
                uploadData.newAlbum = new Album(newAlbumName, newAlbumFolder);
            }

            XmlHelper.SessionStoreUploadData(sessionXml, uploadData);

            DialogResult = DialogResult.OK;
            Close();
        }

        private void uplAlbumsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            Album album  = this.uplAlbumsComboBox.SelectedItem as Album;
            if (album != null)
            {
                bool isRoot = album.IsRoot();
                uplAlbumNewCheckBox.Checked = isRoot;
                uplAlbumNewCheckBox.Enabled = !isRoot;
            }
        }

        private void uplAlbumNewCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            bool isChecked = uplAlbumNewCheckBox.Checked;
            bool isAuto = uplAlbumAutoCheckBox.Checked;

            uplAlbumNameLabel.Enabled = isChecked;
            uplAlbumNameTextBox.Enabled = isChecked;
            uplAlbumFolderLabel.Enabled = isChecked;
            uplAlbumFolderTextBox.Enabled = isChecked && !isAuto;
            uplAlbumAutoCheckBox.Enabled = isChecked;
            //uplAlbumPublishCheckBox.Enabled = isChecked;
        }

        private void uplAlbumAutoCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            bool isAuto = uplAlbumAutoCheckBox.Checked;
            uplAlbumFolderTextBox.Enabled = !isAuto;
            if (isAuto)
            {
                uplAlbumNameTextBox.TextChanged += uplAlbumNameTextBox_TextChanged;
            }
            else
            {
                uplAlbumNameTextBox.TextChanged -= uplAlbumNameTextBox_TextChanged;
            }
        }

        private void uplAlbumNameTextBox_TextChanged(object sender, EventArgs e)
        {
            Album album  = this.uplAlbumsComboBox.SelectedItem as Album;
            if (album != null)
            {
                bool isRoot = album.IsRoot();
                if (isRoot) 
                {
                    uplAlbumFolderTextBox.Text = uplAlbumNameTextBox.Text;
                }
                else
                {
                    uplAlbumFolderTextBox.Text = album.Folder + "/" + uplAlbumNameTextBox.Text;
                }
            }
        }

        private void uplPrivacyLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            privReturnPanel = ConfigPanel.Upload;
            ShowPanel(ConfigPanel.PrivacyPolicy);
        }

        ///////////////////////////////////////////////////////////////////////
        // Privacy Panel functions
        ///////////////////////////////////////////////////////////////////////
        
        private void priCloseButton_Click(object sender, EventArgs e)
        {
            if (privReturnPanel == ConfigPanel.Publish)
            {
                ShowPublishPanel();
            }
            if (privReturnPanel == ConfigPanel.Upload)
            {
                ShowUploadPanel();
            }
        }
    }
}
