﻿/** 
 *   Copyright 2008 Olivian Daniel Tofan
 *   Additions made in 2010 by Robert Tomek
 * 
 *   This file is part of WLPGZenphotoPlugin.
 *
 *   WLPGZenphotoPlugin is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.Xml;

namespace WLPGZenphotoPlugin
{
    /// <summary>
    /// The Publish Plugin API communicates primarily via two XmlDocuments.
    /// The first XmlDocument is the sessionXml. This document will look 
    /// something like the following: (elements preceeded by *s are created
    /// and used soley by this sample plug-in, everything else is provided 
    /// by the Plugin Framework)
    /// 
    /// <?xml version="1.0"?>
    /// <PhotoGalleryPublishSession versionMajor="1" versionMinor="0">
    ///     <PublishParameters>
    ///         <MaxWidth>1024</MaxWidth>
    ///         <MaxHeight>1024</MaxHeight>
    ///         *<Session Name="" Url="" Username="" Password="" />
    ///     </PublishParameters>
    ///     <ItemSet>
    ///         <Item id="16">
    ///             <FullFilePath>C:\Users\Public\Pictures\Sample Pictures\Tree.jpg</FullFilePath>
    ///             <OriginalFileName>Tree.jpg</OriginalFileName>
    ///             <OriginalFileExtension>.jpg</OriginalFileExtension>
    ///             <PerceivedType>image</PerceivedType>
    ///             <Title>Sheep under a tree near Dorset, England.</Title>
    ///             <OriginalWidth>1024</OriginalWidth>
    ///             <OriginalHeight>768</OriginalHeight>
    ///             <LengthMS>0</LengthMS>
    ///             <FileSize>770042</FileSize>
    ///             <KeywordSet>
    ///                 <Keyword>Sample</Keyword>
    ///                 <Keyword>Landscape</Keyword>
    ///             </KeywordSet>
    ///         </Item>
    ///         <Item id="3">
    ///             <FullFilePath>C:\Users\Public\Videos\Sample Videos\Lake.wmv</FullFilePath>
    ///             <OriginalFileName>Lake.wmv</OriginalFileName>
    ///             <OriginalFileExtension>.wmv</OriginalFileExtension>
    ///             <PerceivedType>video</PerceivedType>
    ///             <Title/>
    ///             <OriginalWidth>720</OriginalWidth>
    ///             <OriginalHeight>480</OriginalHeight>
    ///             <LengthMS>0</LengthMS>
    ///             <FileSize>2981738</FileSize> 
    ///             <KeywordSet>
    ///                 <Keyword>Sample</Keyword>
    ///                 <Keyword>Landscape</Keyword>
    ///                 <Keyword>Wildlife</Keyword>
    ///             </KeywordSet>
    ///         </Item>
    ///     </ItemSet>
    ///     *<Log>
    ///         *<Action>This is log entry of a successful publish action</Action>
    ///         *<Error>This is a log entry of a publish error</Error>
    ///     *</Log>
    /// </PhotoGalleryPublishSession>    
    /// 
    /// This XmlDocument will be preserved through the lifetime of the publish 
    /// session and you may make changes to it in any way you wish. This sample 
    /// plugin makes full use of the sessionXml XmlDocument. The static 
    /// functions that begin with Session... perform various operations on the 
    /// sessionXml XmlDocument.
    /// 
    /// The other XmlDocument is the persistXml and it will initially be empty. 
    /// The format you use for this document is up to you. All data stored in 
    /// the persistXml XmlDocument will be encrypted and stored on the local 
    /// machine between publish sessions. The XmlDocument will be loaded and 
    /// decrypted and presented to subsequent publish sessions. This sample 
    /// will store data in the persistXml XmlDocument something like the 
    /// following:
    /// 
    /// <?xml version="1.0"?>
    /// <Settings>
    ///     <Sessions>
    ///         <Session Name="" Url="" Username="" Password="" />
    ///     </Sessions>
    ///     <Defaults Name=""/>
    /// </Settings>    
    /// 
    /// The static functions that begin with Persist... perform various 
    /// operations on the persistXml XmlDocument.
    /// </summary>
    internal class XmlHelper
    {
        /// <summary>
        /// This class consists of only static helper functions
        /// </summary>
        private XmlHelper()
        {
        }

        /// <summary>
        /// This function initializes the persistXml XmlDocument like this:
        /// 
        /// <?xml version="1.0"?>
        /// <Settings>
        ///     <Sessions/>
        ///     <Defaults/>
        /// </Settings>    
        /// 
        /// </summary>
        public static void PersistInitializeDocument(XmlDocument persistXml)
        {
            persistXml.RemoveAll();
            persistXml.AppendChild(persistXml.CreateProcessingInstruction("xml", "version=\"1.0\""));
            XmlNode root = persistXml.CreateElement("Settings");
            persistXml.AppendChild(root);
            root.AppendChild(persistXml.CreateElement("Sessions"));
            root.AppendChild(persistXml.CreateElement("Defaults"));
        }

        /// <summary>
        /// This function loads the Zenphoto Sessions from the persistXml XmlDocument. 
        /// The persisteXml XmlDocument looks something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <Settings>
        ///     <Sessions>
        ///         <Session Name="" Host="" Port="" Directory="" Username="" Password="" />
        ///     </Sessions>
        /// </Settings>    
        /// 
        /// </summary>
        /// 
        /// <remarks>
        /// This function does not assume you have called PersistInitializeDocument. 
        /// </remarks>
        ///

        public static bool PersistHasSessions(XmlDocument persistXml)
        {
            XmlElement element = persistXml.SelectSingleNode("/Settings/Sessions/Session") as XmlElement;
            return (element != null);
        }

        public static bool PersistExistsSession(XmlDocument persistXml, string name)
        {
            XmlNode node = persistXml.SelectSingleNode("/Settings/Sessions");
            XmlElement element = node.SelectSingleNode(string.Format("Session[@Name='{0}']", name)) as XmlElement;
            return (element != null);
        }

        /// <summary>
        /// This function loads the Sessions from the persistXml XmlDocument.
        /// The persisteXml XmlDocument will look something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <Settings>
        ///     <Sessions>
        ///         <Session Name="" ... />
        ///     </Sessions>
        ///     ...
        /// </Settings>    
        ///  
        /// </summary>
        public static void PersistLoadSessions(XmlDocument persistXml, out List<Session> sessions)
        {
            sessions = new List<Session>();
            XmlNode set = persistXml.SelectSingleNode("/Settings/Sessions");
            XmlNodeList list = set.SelectNodes("Session");
            foreach (XmlNode node in list)
            {
                XmlElement element = node as XmlElement;
                Session session = createSession(element);
                sessions.Add(session);
            }
        }

        /// <summary>
        /// This function updates a Session entry in the persistXml XmlDocument.
        /// The persistXml XmlDocument will look something like this:
        ///
        /// <?xml version="1.0"?>
        /// <Settings>
        ///     <Sessions>
        ///         <Session Name="" ... />
        ///     </Users>
        ///     ...
        /// </Settings>    
        /// 
        /// </summary>
        /// 
        /// <remarks>
        /// This function assumes you have called PersistInitializeDocument to create the initial XmlDocument
        /// layout. 
        /// </remarks>
        public static void PersistUpdateSession(XmlDocument persistXml, Session session)
        {
            XmlNode node = persistXml.SelectSingleNode("/Settings/Sessions");
            XmlElement element = node.SelectSingleNode(string.Format("Session[@Name='{0}']", session.Name)) as XmlElement;
            if (element == null)
            {
                element = persistXml.CreateElement("Session");
                node.AppendChild(element);
            }

            element.SetAttribute("Name", session.Name);
            element.SetAttribute("Url", session.Url);
            element.SetAttribute("Username", session.Username);
            element.SetAttribute("Password", session.Password);
        }

        public static void PersistLoadSession(XmlDocument persistXml, string name, out Session session)
        {
            XmlNode node = persistXml.SelectSingleNode("/Settings/Sessions");
            XmlElement element = node.SelectSingleNode(string.Format("Session[@Name='{0}']", name)) as XmlElement;
            if (element == null)
            {
                session = null;
                return;
            }

            session = createSession(element);
        }

        /// <summary>
        /// This function gets the selected token in the persistXml XmlDocument.
        /// The persistXml XmlDocument will look something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <Settings>
        ///     ...
        ///     <Defaults Name=""/>
        ///     ...
        /// </Settings>    
        ///  
        /// </summary>
        ///
        /// <remarks>
        /// This function assumes you have called PersistInitializeDocument to create the initial XmlDocument
        /// layout. 
        /// </remarks>
        public static void PersistLoadSelectedSession(XmlDocument persistXml, out string name)
        {
            name = string.Empty;
            XmlElement element = persistXml.SelectSingleNode("/Settings/Defaults") as XmlElement;
            if (element.HasAttribute("Name"))
            {
                name = element.GetAttribute("Name");
            }
        }

        /// <summary>
        /// This function sets the selected token in the persistXml XmlDocument.
        /// The persistXml XmlDocument will look something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <Settings>
        ///     ...
        ///     <Defaults Name=""/>
        ///     ...
        /// </Settings>    
        ///  
        /// </summary>
        /// 
        /// <remarks>
        /// This function assumes you have called PersistInitializeDocument to create the initial XmlDocument
        /// layout. 
        /// </remarks>
        public static void PersistStoreSelectedSession(XmlDocument persistXml, string name)
        {
            XmlElement element = persistXml.SelectSingleNode("/Settings/Defaults") as XmlElement;
            if (name == string.Empty)
            {
                element.RemoveAttribute("Name");
            }
            else
            {
                element.SetAttribute("Name", name);
            }
        }

        /// <summary>
        /// This function removes a Session entry in the persistXml XmlDocument.
        /// The persistXml XmlDocument will look something like this:
        /// </summary>
        public static void PersistRemoveSession(XmlDocument persistXml, string name)
        {
            XmlNode node = persistXml.SelectSingleNode(string.Format("/Settings/Sessions/Session[@Name='{0}']", name));
            if (node != null)
            {
                node.ParentNode.RemoveChild(node);
            }
        }

        public static void SessionLoadZenphotoSession(XmlDocument sessionXml, out Session session)
        {
            XmlElement element = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/PublishParameters/Session") as XmlElement;
            session = createSession(element);
        }

        public static void SessionStoreDimensions(XmlDocument sessionXml, int maxWidth, int maxHeight)
        {
            XmlNode node = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/PublishParameters");
            XmlNode paramNode = node.SelectSingleNode("MaxWidth");
            paramNode.InnerText = Convert.ToString(maxWidth);

            paramNode = node.SelectSingleNode("MaxHeight");
            paramNode.InnerText = Convert.ToString(maxHeight);
        }

        public static void SessionStoreZenphotoSession(XmlDocument sessionXml, Session session)
        {
            XmlNode node = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/PublishParameters");
            XmlElement element = node.SelectSingleNode("Session") as XmlElement;
            if (element == null)
            {
                element = sessionXml.CreateElement("Session");
                node.AppendChild(element);
            }

            element.SetAttribute("Name", session.Name);
            element.SetAttribute("Url", session.Url);
            element.SetAttribute("Username", session.Username);
            element.SetAttribute("Password", session.Password);
        }

        private static Session createSession(XmlElement element)
        {
            string name = element.GetAttribute("Name");
            string url = element.GetAttribute("Url");
            string username = element.GetAttribute("Username");
            string password = element.GetAttribute("Password");
            Session session = new Session(name, url);
            session.Username = username;
            session.Password = password;

            return session;
        }

        public static void SessionLoadUploadData(XmlDocument sessionXml, out UploadData uploadData)
        {
            XmlElement element = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/PublishParameters/UploadData") as XmlElement;
            string parentAlbumName = element.GetAttribute("ParentAlbumName");
            string parentAlbumFolder = element.GetAttribute("ParentAlbumFolder");
            string isNewAlbumStr = element.GetAttribute("IsNewAlbum");
            bool isNewAlbum = false;
            bool isPublic = false;
            Album newAlbum = null;
            string isPublicStr = element.GetAttribute("IsPublic");
            isPublic = Boolean.Parse(isPublicStr);
            if (isNewAlbumStr != null && isNewAlbumStr.Length > 0) 
            {
                isNewAlbum = true;
                string newAlbumName = element.GetAttribute("NewAlbumName");
                string newAlbumFolder = element.GetAttribute("NewAlbumFolder");
                newAlbum = new Album(newAlbumName, newAlbumFolder);
            }

            uploadData = new UploadData();
            uploadData.parentAlbum = new Album(parentAlbumName, parentAlbumFolder);
            uploadData.isNewAlbum = isNewAlbum;
            uploadData.isPublic = isPublic;
            uploadData.newAlbum = newAlbum;
        }

        public static void SessionStoreUploadData(XmlDocument sessionXml, UploadData uploadData)
        {
            XmlNode node = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/PublishParameters");
            XmlElement element = node.SelectSingleNode("UploadData") as XmlElement;
            if (element == null)
            {
                element = sessionXml.CreateElement("UploadData");
                node.AppendChild(element);
            }

            element.SetAttribute("ParentAlbumName", uploadData.parentAlbum.Name);
            element.SetAttribute("ParentAlbumFolder", uploadData.parentAlbum.Folder);
            element.SetAttribute("IsPublic", uploadData.isPublic.ToString());
            if (uploadData.isNewAlbum) 
            {
                element.SetAttribute("IsNewAlbum", uploadData.isNewAlbum.ToString());
                element.SetAttribute("NewAlbumName", uploadData.newAlbum.Name);
                element.SetAttribute("NewAlbumFolder", uploadData.newAlbum.Folder);
            }
        }

        /// <summary>
        /// This function loads the information associated with the specified item in the item set from the sessionXml XmlDocument.
        /// The sessionXml XmlDocument looks something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <PhotoGalleryPublishSession versionMajor="12" versionMinor="0">
        ///     ...
        ///     <ItemSet>
        ///         <Item id="1">
        ///             <OriginalFileName>sample1.jpg</OriginalFileName>
        ///             <Title>sample1</Title>
        ///             <KeywordSet>
        ///                 <Keyword>Keyword 1</Keyword>
        ///                 <Keyword>Keyword 2</Keyword>
        ///             </KeywordSet>
        ///             ...
        ///         </Item>
        ///         <Item id="2">
        ///             <OriginalFileName>sample2.jpg</OriginalFileName>
        ///             <Title>sample2</Title>
        ///             <KeywordSet>
        ///                 <Keyword>Keyword 1</Keyword>
        ///                 <Keyword>Keyword 2</Keyword>
        ///             </KeywordSet>
        ///             ...
        ///         </Item>
        ///     </ItemSet>
        ///     ...
        /// </PhotoGalleryPublishSession>    
        /// 
        /// </summary>
        public static bool SessionLoadItemInfo(XmlDocument sessionXml, string mediaObjectId, out PhotoAttributes photoAttributes)
        {
            XmlNode node = sessionXml.SelectSingleNode(string.Format("/PhotoGalleryPublishSession/ItemSet/Item[@id=\'{0}\']", mediaObjectId));
            if (node != null)
            {
                photoAttributes = new PhotoAttributes();
                XmlElement element = node.SelectSingleNode("OriginalFileName") as XmlElement;
                photoAttributes.Filename = element.InnerText;

                element = node.SelectSingleNode("Title") as XmlElement;
                photoAttributes.Title = element.InnerText;

                element = node.SelectSingleNode("KeywordSet") as XmlElement;
                XmlNodeList list = element.SelectNodes("Keyword");
                foreach (XmlNode keyword in list)
                {
                    photoAttributes.Tags.Add(keyword.InnerText);
                }

                return true;
            }
            photoAttributes = null;
            return false;
        }

        /// <summary>
        /// This function gets the count of error log entries in the the sessionXml XmlDocument.
        /// The sessionXml XmlDocument looks something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <PhotoGalleryPublishSession versionMajor="12" versionMinor="0">
        ///     ...
        ///     <Log>
        ///         <Action>This is log entry of a successful publish action</Action>
        ///         <Error>This is a log entry of a publish error</Error>
        ///     </Log>
        /// </PhotoGalleryPublishSession>    
        /// 
        /// </summary>
        public static void SessionGetErrorLogCount(XmlDocument sessionXml, out int count)
        {
            XmlNode node = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/Log");
            XmlNodeList list = node.SelectNodes("Error");
            count = list.Count;
        }

        /// <summary>
        /// This function gets the log entries in the the sessionXml XmlDocument.
        /// The sessionXml XmlDocument looks something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <PhotoGalleryPublishSession versionMajor="12" versionMinor="0">
        ///     ...
        ///     <Log>
        ///         <Action>This is log entry of a successful publish action</Action>
        ///         <Error>This is a log entry of a publish error</Error>
        ///     </Log>
        /// </PhotoGalleryPublishSession>    
        /// 
        /// </summary>
        public static void SessionLoadLog(XmlDocument sessionXml, out List<string> log)
        {
            log = new List<string>();
            XmlElement element = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession/Log") as XmlElement;
            if (element != null)
            {
                foreach (XmlNode childNode in element.ChildNodes)
                {
                    log.Add(childNode.InnerText);
                }
            }
        }

        /// <summary>
        /// This function stores a log entry in the sessionXml XmlDocument.
        /// The sessionXml XmlDocument looks something like this:
        /// 
        /// <?xml version="1.0"?>
        /// <PhotoGalleryPublishSession versionMajor="12" versionMinor="0">
        ///     ...
        ///     <Log>
        ///         <Action>This is log entry of a successful publish action</Action>
        ///         <Error>This is a log entry of a publish error</Error>
        ///     </Log>
        /// </PhotoGalleryPublishSession>    
        /// 
        /// </summary>
        public static void SessionLog(XmlDocument sessionXml, bool fError, string message)
        {
            XmlNode node = sessionXml.SelectSingleNode("/PhotoGalleryPublishSession");
            XmlElement element = node.SelectSingleNode("Log") as XmlElement;
            if (element == null)
            {
                element = sessionXml.CreateElement("Log");
                node.AppendChild(element);
            }

            string name = fError ? "Error" : "Action";
            XmlElement entry = sessionXml.CreateElement(name);
            element.AppendChild(entry);
            entry.InnerText = message;
        }
    }
}
