﻿/** 
 *   Copyright 2008 Olivian Daniel Tofan
 * 
 *   This file is part of WLPGZenphotoPlugin.
 *
 *   WLPGZenphotoPlugin is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Collections.Specialized;
using System.Xml;

namespace WLPGZenphotoPlugin
{
    class ZenphotoHelper
    {
        private static CookieContainer cookieContainer = new CookieContainer();

        private static void ZenphotoPostMultipart(string boundary, NameValueCollection map, Stream stream)
        {
            string formdataTemplate = "\r\n--" + boundary + "\r\nContent-Disposition: form-data; name=\"{0}\";\r\n\r\n{1}";
            foreach (string key in map.Keys)
            {
                string formItem = string.Format(formdataTemplate, key, map[key]);
                byte[] formItemBytes = System.Text.Encoding.UTF8.GetBytes(formItem);
                stream.Write(formItemBytes, 0, formItemBytes.Length);
            }
        }

        private static int ZenphotoPostMultipartStream(string boundary, string streamId, string streamName, Stream inputStream, Stream stream)
        {
            byte[] boundaryBytes = System.Text.Encoding.ASCII.GetBytes("\r\n--" + boundary + "\r\n");
            string headerTemplate = "Content-Disposition: form-data; name=\"{0}\"; filename=\"{1}\"\r\nContent-Type: application/octet-stream\r\n\r\n";
            string header = string.Format(headerTemplate, streamId, streamName);
            byte[] headerBytes = System.Text.Encoding.UTF8.GetBytes(header);
            int length = 0;
            
            stream.Write(headerBytes, 0, headerBytes.Length);
            length += headerBytes.Length;

            byte[] buffer = new byte[1024];
            int bytesRead = 0;
            while ((bytesRead = inputStream.Read(buffer, 0, buffer.Length)) != 0)
            {
                stream.Write(buffer, 0, bytesRead);
                length += bytesRead;
            }

            stream.Write(boundaryBytes, 0, boundaryBytes.Length);
            length += boundaryBytes.Length;

            return length;
        }

        private static void ZenphotoPostUrlencoded(NameValueCollection map, Stream stream)
        {
            string formdataTemplate = "{0}={1}";
            bool first = true;
            foreach (string key in map.Keys)
            {
                string formItem = string.Format(formdataTemplate, key, map[key]);
                if (!first)
                {
                    formItem = "&" + formItem;
                }
                else
                {
                    first = false;
                }
                String formItemEscaped = Uri.EscapeUriString(formItem);
                byte[] formItemBytes = System.Text.Encoding.ASCII.GetBytes(formItemEscaped);
                stream.Write(formItemBytes, 0, formItemBytes.Length);
            }
        }

        private static string GetWebString(Uri uri)
        {
            System.Net.ServicePointManager.Expect100Continue = false;
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(uri);
            request.Method = "GET";
            request.CookieContainer = cookieContainer;
            WebResponse response = request.GetResponse();
            StreamReader reader = new StreamReader(response.GetResponseStream());
            string result = reader.ReadToEnd();
            if (Log.DEBUG)
            {
                Log.Debug(result);
            }
            reader.Close();
            response.Close();
            return result;
        }

        public static bool ZenphotoLogin(Session session)
        {
            bool multipart = true;
            string boundary = "" + DateTime.Now.Ticks.ToString("x");
            byte[] boundaryBytes = System.Text.Encoding.ASCII.GetBytes("\r\n--" + boundary + "\r\n");

            System.Net.ServicePointManager.Expect100Continue = false;

            HttpWebRequest httpWebRequest = (HttpWebRequest) WebRequest.Create(session.GetLoginUri());
            httpWebRequest.Method = "POST";
            httpWebRequest.KeepAlive = true;
            httpWebRequest.Credentials = System.Net.CredentialCache.DefaultCredentials;
            httpWebRequest.Expect = null;
            httpWebRequest.Accept = "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8";
            httpWebRequest.CookieContainer = cookieContainer;
            
            if (multipart)
            {
                httpWebRequest.ContentType = "multipart/form-data; boundary=" + boundary;
            } 
            else 
            {
                httpWebRequest.ContentType = "application/x-www-form-urlencoded";
            }
            
            Stream memStream = new MemoryStream();

            NameValueCollection map = new NameValueCollection();
            map.Add("login", "1");
            map.Add("user", session.Username);
            map.Add("pass", session.Password);
            if (multipart)
            {
                ZenphotoPostMultipart(boundary, map, memStream);
            }
            else
            {
                ZenphotoPostUrlencoded(map, memStream);
            }
            if (multipart)
            {
                memStream.Write(boundaryBytes, 0, boundaryBytes.Length);
            }

            httpWebRequest.ContentLength = memStream.Length;
            Stream requestStream = httpWebRequest.GetRequestStream();    

            memStream.Position = 0;
            byte[] tempBuffer = new byte[memStream.Length];
            memStream.Read(tempBuffer, 0, tempBuffer.Length);
            requestStream.Write(tempBuffer, 0, tempBuffer.Length);
            if (Log.DEBUG)
            {
                memStream.Position = 0;
                StreamReader reader = new StreamReader(memStream);
                Log.Debug(reader.ReadToEnd());
            }
            memStream.Close();
            requestStream.Close();

            HttpWebResponse httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            Stream responseStream = httpWebResponse.GetResponseStream();
            if (Log.DEBUG)
            {
                StreamReader reader = new StreamReader(responseStream);
                Log.Debug(reader.ReadToEnd());
            }

            bool loggedin = false;
            foreach (Cookie cookie in httpWebResponse.Cookies)
            {
                if (cookie.Name == "zenphoto_auth" && cookie.Value != "deleted")
                {
                    loggedin = true;
                }
            }

            httpWebResponse.Close();
            httpWebRequest = null;
            httpWebResponse = null;

            Log.Info("User " + session.Username + (loggedin ? " is " : " is not ") + "logged in");

            return loggedin;
        }

        public static void ZenphotoSendFile(Session session, UploadData uploadData, PhotoAttributes photoAttributes, Stream stream)
        {
            string boundary = "" + DateTime.Now.Ticks.ToString("x");
            byte[] boundaryBytes = System.Text.Encoding.ASCII.GetBytes("\r\n--" + boundary + "\r\n");

            System.Net.ServicePointManager.Expect100Continue = false;

            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(session.GetUploadUri());
            httpWebRequest.Method = "POST";
            httpWebRequest.KeepAlive = true;
            httpWebRequest.Credentials = System.Net.CredentialCache.DefaultCredentials;
            httpWebRequest.Expect = null;
            httpWebRequest.Accept = "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8";
            httpWebRequest.CookieContainer = cookieContainer;
            httpWebRequest.ContentType = "multipart/form-data; boundary=" + boundary;
            
            Stream memStream = new MemoryStream();

            NameValueCollection map = new NameValueCollection();
            map.Add("processed", "1");
            map.Add("albumselect", uploadData.parentAlbum.Folder);
            map.Add("existingfolder", uploadData.isNewAlbum.ToString().ToLower());
            map.Add("publishalbum", uploadData.isPublic ? "1" : "0");
            if (uploadData.isNewAlbum)
            {
                map.Add("newalbum", "on");
                map.Add("albumtitle", uploadData.newAlbum.Name);
                map.Add("folder", uploadData.newAlbum.Folder);
                map.Add("folderdisplay", uploadData.newAlbum.Folder);
            }
            else
            {
                map.Add("folder", uploadData.parentAlbum.Folder);
            }
            
            ZenphotoPostMultipart(boundary, map, memStream);
            memStream.Write(boundaryBytes, 0, boundaryBytes.Length);
            ZenphotoPostMultipartStream(boundary, "files[]", photoAttributes.Filename, stream, memStream);
            
            httpWebRequest.ContentLength = memStream.Length;
            Stream requestStream = httpWebRequest.GetRequestStream();

            memStream.Position = 0;
            byte[] tempBuffer = new byte[memStream.Length];
            memStream.Read(tempBuffer, 0, tempBuffer.Length);
            requestStream.Write(tempBuffer, 0, tempBuffer.Length);
            if (Log.DEBUG)
            {
                memStream.Position = 0;
                StreamReader reader = new StreamReader(memStream);
                Log.Debug(reader.ReadToEnd());
            }
            memStream.Close();
            requestStream.Close();

            HttpWebResponse httpWebResponse = (HttpWebResponse)httpWebRequest.GetResponse();
            Stream responseStream = httpWebResponse.GetResponseStream();
            if (Log.DEBUG)
            {
                StreamReader reader = new StreamReader(responseStream);
                Log.Debug(reader.ReadToEnd());
            }

            httpWebResponse.Close();
            httpWebRequest = null;
            httpWebResponse = null;
        }


        public static void ZenphotoGetAlbums(Session session, out List<Album> albums)
        {
            /**
             * Ugly implementation. This is a workaround... 
             * 
             * XmlDocument.Load failed because the returned string is not XHTML.
             * Proceess the returned string till is valid XHTML.
             * (eg. java script code, some missspell og nbsp)
             */
            albums = new List<Album>();
            try
            {
                string result = GetWebString(session.GetAlbumsUri());
                int beginOffset = result.IndexOf("<select id=\"albumselectmenu\"");
                int endOffset = result.IndexOf("</select>", beginOffset) + "</select>".Length;
                string xml = result.Substring(beginOffset, endOffset - beginOffset);
                xml = xml.Replace("&nbsp;", "");
                xml = xml.Replace("&raquo;", "");
                XmlReaderSettings settings = new XmlReaderSettings();
                settings.XmlResolver = null;
                settings.ValidationType = ValidationType.None;
                StringReader strReader = new StringReader(xml);
                XmlReader xmlReader = XmlTextReader.Create(strReader, settings);

                XmlDocument xmlResult = new XmlDocument();
                xmlResult.Load(xmlReader);

                XmlNode node = xmlResult.SelectSingleNode("//*[@id=\"albumselectmenu\"]");
                if (node != null)
                {
                    XmlNodeList list = node.SelectNodes("option");
                    foreach (XmlNode optionNode in list)
                    {
                        XmlElement optionElement = optionNode as XmlElement;
                        string albumFolder = optionElement.GetAttribute("value");
                        //string albumName = optionElement.InnerText;
                        //Log.Info("ALBUM - name: " + albumName + " folder: " + albumFolder);
                        Album album = new Album(albumFolder);
                        albums.Add(album);
                    }
                }
            }
            catch (Exception ex)
            {
                Log.Error(ex);
            }
        }
    }
}
