﻿/** 
 *   Copyright 2008 Olivian Daniel Tofan
 * 
 *   This file is part of WLPGZenphotoPlugin.
 *
 *   WLPGZenphotoPlugin is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.WindowsLive.PublishPlugins;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using System.Threading;

namespace WLPGZenphotoPlugin
{
    /// <summary>
    /// The publishing plug-in platform communicates with a plug-in through the IPublishPlugin 
    /// interface, which the plug-in must implement. Each selected photo or videos file is passed 
    /// to the plug-in as a stream; metadata can be read through the IPublishProperties interface. 
    /// Publishing progress can be displayed to the user through the IPublishProgressCallback 
    /// interface.
    /// 
    /// The hosting Windows Live application discovers installed publishing plug-ins by enumerating 
    /// registry keys in the following location:
    /// 
    ///     HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows Live\PublishPlugins
    ///     
    /// The name of a publishing plug-in's key is not visible to the end user. Each key corresponds 
    /// to one plug-in and contains the following values:
    /// 
    ///     AssemblyPath - The full path to your plug-in assembly. REG_SZ; required.
    ///     ClassName - The fully qualified class name of your plug-in, including any namespaces. 
    ///         REG_SZ; required.
    ///     FriendlyName - The a display name for the plug-in; this value is displayed on the 
    ///         Publish menu. REG_SZ; required.
    ///     IconPath - The path and resource ID for an icon to display with your plug-in. REG_SZ; 
    ///         optional.
    /// 
    /// The following provides a sample plug-in registry entry, using "Your Publishing Plug-In" as 
    /// an example.
    /// 
    ///     [HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows Live\PublishPlugins\YourPublishingPlugin]
    ///     "AssemblyPath"="C:\\Program Files\\YourPublishingPlugin\\YourPublishingPlugIn.dll"
    ///     "ClassName"="YourPublishingPlugin.PublishPlugin"
    ///     "FriendlyName"="Your Publishing Plug-in"
    ///     "IconPath"="C:\\Program Files\\YourPublishingPlugin\\YourPublishingPlugIn.dll,-32512"
    /// </summary>
    public class ZenphotoPlugin: IPublishPlugin
    {
        private static readonly string uploadFailureLogFormat = "ERROR: {0} failed to upload with the following error:\r\n\t{1}";
        private static readonly string uploadSuccessLogFormat = "{0} ({1}) uploaded successfully";

        /// <summary>
        /// Directs the publishing plug-in to show its configuration UI.
        /// </summary>
        /// <param name="parentWindow">
        /// A handle to the parent window that will be used to display any UI to the user.
        /// </param>
        /// <param name="sessionXml">
        /// Session specific information for this publishing session.
        /// </param>
        /// <param name="persistXml">
        /// Persistent information for this plug-in.
        /// </param>
        /// <param name="publishProperties">
        /// A reference an IPublishProperties interface, which can be used to access a media 
        /// object's metadata store.
        /// </param>
        /// <returns>
        /// true if the publishing operation should continue; false if there is an error, or the 
        /// user cancelled the publishing operation.
        /// </returns>
        /// <remarks>
        /// ShowConfigurationSettings is called by the hosting Windows Live application at the 
        /// beginning of each publishing session; this enables the publishing plug-in to request 
        /// configuration information from the user. If ShowConfigurationSettings returns true, the 
        /// contents of the XmlDocument specified by persistXml are saved, and will be provided to 
        /// future calls to ShowConfigurationSettings. If ShowConfigurationSettings returns false, 
        /// the contents of persistXml are not saved.
        /// </remarks>
        public bool ShowConfigurationSettings(IWin32Window parentWindow, XmlDocument sessionXml, XmlDocument persistXml, IPublishProperties publishProperties)
        {
            // Show the configuration dialog to the user and return the result.
            try
            {
                return ConfigForm.GetSessionInfo(parentWindow, sessionXml, persistXml);
            }
            catch (Exception ex)
            {
                Log.Error(ex);
                return false;
            }
        }

        /// <summary>
        /// Directs the publishing plug-in to publish a specified media object.
        /// </summary>
        /// <param name="parentWindow">
        /// A handle to the parent window that will be used to display any UI to the user.
        /// </param>
        /// <param name="mediaObjectId">
        /// The ID of the media object to be published. Corresponds to the "id" attribute of the 
        /// Item element within the sessionXml parameter.
        /// </param>
        /// <param name="stream">
        /// The contents of the media object to be published.
        /// </param>
        /// <param name="sessionXml">
        /// Session specific information for this publish session.
        /// </param>
        /// <param name="publishProperties">
        /// A reference to an IPublishProperties interface, which can be used to access a media 
        /// object's metadata store.
        /// </param>
        /// <param name="callback">
        /// A reference to an IPublishProgressCallback callback interface that can be used to 
        /// communicate progress information to the user.
        /// </param>
        /// <param name="cancelEvent">
        /// A handle to a thread synchronization event.
        /// </param>
        /// <returns>
        /// true if the specified media object was successfully published; otherwise, false.
        /// </returns>
        /// <remarks>
        /// The PublishItem method is called once for each media item that is to be published 
        /// during a particular publishing session. A return value of false will not abort a job 
        /// that is publishing multiple media objects. However, if a publishing plug-in raises an 
        /// unhandled exception, the hosting Windows Live application will abort the entire 
        /// publishing session. After a publishing job completes, the total number of successfully 
        /// published items is displayed to the user; the number of successfully published items 
        /// corresponds to the number of times the PublishItem method is called and returns true. 
        /// A plug-in that publishes items asynchronously can wait on cancelEvent to detect and 
        /// react to user-initiated cancellation requests.
        /// </remarks>
        public bool PublishItem(IWin32Window parentWindow, string mediaObjectId, Stream stream, XmlDocument sessionXml, IPublishProperties publishProperties, IPublishProgressCallback callback, EventWaitHandle cancelEvent)
        {
            try
            {
                Session session;
                PhotoAttributes photoAttributes;
                XmlHelper.SessionLoadZenphotoSession(sessionXml, out session);
                XmlHelper.SessionLoadItemInfo(sessionXml, mediaObjectId, out photoAttributes);

                Log.Debug("PublishItem " + mediaObjectId);
                Log.Debug("Session " + session.ToString());
                Log.Debug("\tTitle: " + photoAttributes.Title);
                Log.Debug("\tFilename: " + photoAttributes.Filename);
                Log.Debug("\tDescription: " + photoAttributes.Description);

                try
                {
                    UploadData uploadData;
                    XmlHelper.SessionLoadUploadData(sessionXml, out uploadData);
                    ZenphotoHelper.ZenphotoSendFile(session, uploadData, photoAttributes, stream);
                    XmlHelper.SessionLog(sessionXml, false, string.Format(uploadSuccessLogFormat, photoAttributes.Filename, photoAttributes.Id));
                }
                catch (Exception e)
                {
                    Log.Error(e);
                    XmlHelper.SessionLog(sessionXml, true, string.Format(uploadFailureLogFormat, photoAttributes.Filename, e.Message));
                    return false;
                }

                callback.SetPublishProgress(100);
                return true;
            }
            catch (Exception ex)
            {
                Log.Error(ex);
                return false;
            }
        }

        /// <summary>
        /// Indicates whether or not a publishing plug-in has summary information available to 
        /// display to the user.
        /// </summary>
        /// <param name="sessionXml">
        /// Session specific information for this publish session.
        /// </param>
        /// <returns>
        /// true if the publishing plug-in has summary information to display; otherwise, false.
        /// </returns>
        /// <remarks>
        /// HasSummaryInformation is used to determine whether or not a More Information link 
        /// appears on the summary dialog that is displayed to the user after a publishing job 
        /// completes. If this method returns true, a More Information link appears on the summary 
        /// dialog; if this method returns false, a More Information link will not be shown on the 
        /// summary dialog. Clicking the More Information link causes ShowSummaryInformation to be 
        /// called. Note that if HasSummaryInformation returns false, ShowSummaryInformation is 
        /// never called.
        /// </remarks>
        public bool HasSummaryInformation(XmlDocument sessionXml)
        {
            return true;
        }

        /// <summary>
        /// Directs the publishing plug-in to display summary information.
        /// </summary>
        /// <param name="parentWindow">
        /// A handle to the parent window that will be used to display the summary information to 
        /// the user.
        /// </param>
        /// <param name="sessionXml">
        /// Session specific information for this publish session.
        /// </param>
        /// <remarks>
        /// ShowSummaryInformation is called when the More Information link on the summary dialog 
        /// is clicked. The return value of HasSummaryInformation determines whether or not a More 
        /// Information link is displayed. Note that if HasSummaryInformation returns false, 
        /// ShowSummaryInformation is never called.
        /// </remarks>
        public void ShowSummaryInformation(IWin32Window parentWindow, XmlDocument sessionXml)
        {
            try
            {
                StatusForm.DisplayResults(parentWindow, sessionXml);
            } 
            catch (Exception ex)
            {
                Log.Error(ex);
            }
        }

        /// <summary>
        /// Indicates whether or not a publishing plug-in has publishing results available to 
        /// display to the user.
        /// </summary>
        /// <param name="sessionXml">
        /// Session specific information for this publish session.
        /// </param>
        /// <returns>
        /// true if the publishing plug-in has publishing results to display; otherwise, false.
        /// </returns>
        /// <remarks>
        /// HasPublishResults is used to determine whether or not a View button appears on the 
        /// summary dialog that is displayed to the user after a publishing job completes. If this 
        /// method returns true, a View button appears on the summary dialog; if this method 
        /// returns false, a View button will not be shown on the summary dialog. Clicking the View 
        /// button causes LaunchPublishResults to be called. Note that if HasPublishResults returns 
        /// false, LaunchPublishResults is never called.
        /// </remarks>
        public bool HasPublishResults(XmlDocument sessionXml)
        {
            return false;
        }

        /// <summary>
        /// Directs the publishing plug-in to display publishing results.
        /// </summary>
        /// <param name="sessionXml">
        /// Session specific information for this publish session.
        /// </param>
        /// <remarks>
        /// LaunchPublishResults is called when the View button on the summary dialog is clicked. 
        /// The return value of HasPublishResults determines whether or not a View button is 
        /// displayed. Note that if HasPublishResults returns false, LaunchPublishResults is never 
        /// called.
        /// </remarks>
        public void LaunchPublishResults(XmlDocument sessionXml)
        {
            
        }
    }
}
